import { system, world } from "@minecraft/server";
import { MessageFormData, ModalFormData, ActionFormData} from "@minecraft/server-ui";

/**
 * Fires 1 second delay,
 * @param {() => void} callback
 * @param {number} tick
 * @example setTickTimeout(() => { /* code *\/ }, 20)
*/
export function setTickTimeout(callback, tick) {
    const id = system.runInterval(() => {
      callback()
      system.clearRun(id)
    }, tick)
}

export async function scoreboard(name='', value=0, obj='debug', oper='set') {
    await runcmd(`scoreboard players ${oper} "${name}" ${obj} ${value}`,true)
}

function isNotSelector (text){
    return text.indexOf('@') == -1
}

export function getScore(objective, target, useZero = false) {
    try {
        const oB = world.scoreboard.getObjective(objective)
        if (typeof target == 'string') return oB.getScore(oB.getParticipants().find(pT => pT.displayName == target))
        return oB.getScore(target.scoreboard)
    } catch {
        return useZero ? 0 : NaN
    }
}

export function BoolToNum(boolen){
  return boolen == true ? 1 : 0
}

export function getItemAmounts(player, items) {
  const inv = player.getComponent('inventory')?.container;
  if (!inv) return;
  const counts = {};
  for (let id of items) {
    counts[id] = 0;
  }
  for (let i = 0; i < inv.size; i++) {
    const item = inv.getItem(i);
    if (!item) continue;
    const itemId = item.typeId.replace('minecraft:', '')
    const id = items.find(v => itemId == v.replace('minecraft:', ''))
    if (id) counts[id] += item.amount;
  }
  return counts;
}

export async function runfunction(fname){
    await runcmd(`function ${fname}`, true)
}


export function randomInt(min, max) { 
   return Math.floor(Math.random() * (max - min + 1) + min)
}

export async function runcmd(command, need_log = false){
    try{
        await world.getDimension('overworld').runCommandAsync(command)
    }catch(error){
        if (need_log) {
            console.error(`CMD: ${command}\nError: ${error}`)
        }
    }
}

export async function runcmd_at(name, command, need_log=false){
    if (isNotSelector(name)) {
        await runcmd(`execute as "${name}" at @s positioned as "${name}" run ${command}`,need_log)
    } else {
        await runcmd(`execute as ${name} at @s positioned as @s run ${command}`, need_log)
    }
}
export async function playsound(sound, name='@a', volume = 1, pitch = 1){
    if (isNotSelector(name)) {
        await runcmd(`execute as "${name}" at "${name}" run playsound ${sound} @s ~~~ ${volume} ${pitch}`)
    } else {
        await runcmd(`execute as ${name} at @s run playsound ${sound} @s ~~~ ${volume} ${pitch}`)
    }
  }

/**
 * Send raw text (@selector support)
 * @param {JSON} rawtext
 * @param {nameTag} player
 */
export async function tellraw(rawtext, name='@a'){
    if (typeof rawtext == "object") {
        rawtext = JSON.stringify(rawtext)
    }

	if (isNotSelector(name)) {
        runcmd_at(name,`tellraw "${name}" ${rawtext}`,true)
    } else {
    	runcmd_at(name,`tellraw ${name} ${rawtext}`,true)
    }
}

/**
 * Send raw text with one element
 * @param {Text} text
 * @param {nameTag} player
 * @param {['text', 'translate']} key
 */
export function rawtext(text, name='@a', key='text',need_log=false){
	if (isNotSelector(name)) {
      runcmd(`tellraw "${name}" {"rawtext":[{"${key}":"${text}"}]}`,need_log)
    } else {
    	runcmd(`tellraw ${name} {"rawtext":[{"${key}":"${text}"}]}`,need_log)
    }
}

export function hasTag(target, tag){
  return target.getTags().indexOf(tag) != -1
}

export function textTag(target, tag, text_true, text_false){
  return hasTag(target, tag) ? text_true : text_false
}